/* sort chinese-english strings according to pin-yin in a LaTeX index file */

#if !defined(__LARGE__) && !defined (__COMPACT__) && !defined (__HUGE__)
  #error "This program should be compiled using large data models"
#endif

typedef unsigned char byte;

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>

#define NbrOfCC		6768
static byte *pytable[NbrOfCC];

typedef struct {
  byte *original;
  byte *cleaned;
  byte *pinyin;
  int  pageno;
} ccstring;

#define MaxLines	2500
static ccstring list[MaxLines];

void near clean(byte *s, byte *ss, byte *py);
int cccomp(ccstring *s1, ccstring *s2);

void main(int argc, byte *argv[])
{
  register int i,j,n;
  FILE *f_in;
  byte s[256], s1[512], s2[2048], *p, *p1, *p2, ch;

  if (argc!=2) {
usage:
    fprintf(stderr,"Usage: ccsort inputfile [>outputfile]\n");
    return;
  }

  /* read in pytable */
  fprintf(stderr,"Reading pinyin table...");
  strcpy(s,argv[0]); strcpy(s+strlen(s)-3,"DAT");
  if ((f_in=fopen(s,"rt"))==NULL) {
    fprintf(stderr,"Can't open file \"%s\"!\n",s);
    return;
  }
  for (i=0; i<NbrOfCC; i++) {
    fgets(s,256,f_in);
    if (strlen(s)<3) strcpy(s,"   zzzzzzz");
    if ((pytable[i]=malloc(strlen(s)-2))==NULL) {
mem_err:
      fprintf(stderr,"\nMemory overflow!\n");
      fclose(f_in); return;
    }
    strcpy(pytable[i],s+3);
  }
  fclose(f_in);
  fprintf(stderr,"Done.\n");

  if ((f_in=fopen(argv[1],"rt"))==NULL) goto usage;

  n=0;
  fprintf(stderr,"Reading input file...");
  while (fgets(s,256,f_in)!=NULL) {
    p=s+strlen(s)-1;
    while (isspace(*p) && p>s) p--;
    *(++p)='\0';
    if (!s[0] || (isspace(s[0]) && !s[1])) continue;
    if (strncmp(s,"\\indexentry{",12)) continue;
    /* retrieve the page number */
    s[strlen(s)-1]='\0';
    p=s+strlen(s)-1;
    while (*p != '{' && p>s) p--;
    if (*p!='{') continue;
    j=atoi(p+1);
    *p='\0';
    if (n>=MaxLines) {
      fprintf(stderr,"\nWarning: only first %d input lines read.\n",MaxLines);
      break;
    }
    clean(s,s1,s2);
    p=malloc(strlen(s)+1);
    p1=malloc(strlen(s1)+1);
    p2=malloc(strlen(s2)+1);
    if (p==NULL || p1==NULL || p2==NULL) goto mem_err;
    strcpy(list[n].original=p, s);
    strcpy(list[n].cleaned=p1, s1);
    strcpy(list[n].pinyin=p2, s2);
    list[n].pageno=j;
    n++;
  }
  fclose(f_in);
  fprintf(stderr,"Done.\n");

  fprintf(stderr,"Sorting...");
  qsort(list, n, sizeof(ccstring), cccomp);
  fprintf(stderr,"Done.\n");

  /* output results */
  ch='\0';
  for (i=0; i<n; ) {
    int lastpageno,printedpageno;
    j=toupper(list[i].pinyin[0]);
    if (j!=ch) {
      printf("\n\\section{%c}\n\n",ch=j);
    }
    printf("%s{%d", list[i].original, list[i].pageno);
    lastpageno=printedpageno=list[i].pageno;
    for (j=i+1; j<n; j++) {
      list[i].pageno=list[j].pageno;	/* ignore pageno when comparing */
      if (cccomp(list+i, list+j)) break;
      /* don't print duplicate page numbers,
         and replace successive page numbers by a range */
      if (lastpageno!=list[j].pageno && lastpageno+1!=list[j].pageno) {
        if (lastpageno!=printedpageno) printf("--%d",lastpageno);
        printf(", %d",printedpageno=list[j].pageno);
      }
      lastpageno=list[j].pageno;
    }
    if (lastpageno!=printedpageno) printf("--%d",lastpageno);
    printf("}\n");
    i=j;
  }

  return;
}

void near clean(byte *s, byte *ss, byte *py)
/* put cleaned string in "ss", pin yin in "py" */
{
  register byte *p, *q;
  byte t[20];
  int code;

  p=ss; q=s;
  while (*q) {
    if (*q>160) {
      if (!q[1]) break;
      if (q[0]==161 && q[1]==161) {q+=2; continue;}
      *(p++)=' '; *(p++)=*(q++); *(p++)=*(q++); *(p++)=' ';
    } else switch (*q) {
      case '\\' :
        for (q++; (*q>='a' && *q<='z') || (*q>='A' && *q<='Z'); q++);
        break;
/* ----------------------------------------
        case '{'  :
      case '}'  :
        q++; *(p++)=' ';
        break;
      default :
        *(p++)=*(q++);
        break;
---------------------------------------- */
      default :
        if ( (*q>='a' && *q<='z') || (*q>='A' && *q<='Z') ||
             (*q>='0' && *q<='9') ) {
          *(p++)=*(q++);
        } else {
          /* ignore special chars */
          q++; *(p++)=' ';
        }
        break;
    }
  }
  *p='\0';

  /* remove repeated spaces from ss */
  p=ss;
  while (*p) {
    if (isspace(*p)) {
      q=p+1;
      while (isspace(*q)) q++;
      strcpy(p+1,q);
    }
    p++;
  }
  /* remove leading and trailing spaces */
  p=ss;
  while (isspace(*p)) p++;
  strcpy(ss,p);
  p=ss+strlen(ss)-1;
  while (isspace(*p)) p--;
  *(++p)='\0';

  /* transform Hanzi to py and save in "py" */
  q=ss; p=py;
  while (*q) {
    if (*q<161) *(p++)=*(q++); else {
      code=(q[0]-161)*94+(q[1]-161);
      q+=2;
      if (code>=1410 && code<NbrOfCC+1410) {
        sscanf(pytable[code-1410],"%s",t);
        strcpy(p,t);
        p+=strlen(t);
      }
    }
  }
  *p='\0';

  return;
}

int cccomp(ccstring *s1, ccstring *s2)
/* used by qsort */
{
  register int i;

  i=stricmp(s1->pinyin,s2->pinyin);
  if (i) return i;
  i=stricmp(s1->cleaned,s2->cleaned);
  if (i) return i;
  return s1->pageno - s2->pageno;
}
